/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AbstractDroolsWorkingMemory.java 2693 2008-10-30 13:12:22Z loris $
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.rules;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.drools.FactException;
import org.drools.FactHandle;
import org.drools.RuleBase;
import org.drools.WorkingMemory;
import org.ow2.jasmine.rules.api.IDroolsWorkingMemory;

public abstract class AbstractDroolsWorkingMemory implements
        IDroolsWorkingMemory {

    /**
     * Logger.
     */
    private static Logger logger = null;

    /**
     * Thread to fire rules.
     */
    protected Thread thread = null;

    /**
     * True if the thread is started.
     */
    protected volatile boolean started;

    /**
     * Period to fire rules.
     */
    private volatile long timeout = 5000;

    /**
     * A directory to save generated files.
     */
    protected File dumpDir;

    /**
     * The managed working memory.
     */
    protected volatile WorkingMemory workingMemory = null;

    /**
     * {@link ClassLoader} to build rules.
     */
    protected final ClassLoader classLoader;

    protected RuleBase ruleBase;

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    public AbstractDroolsWorkingMemory(final ClassLoader classloader, final Logger logger) {
        classLoader = classloader;
        AbstractDroolsWorkingMemory.logger = logger;
    }

    // ------------------------------------------------------------------------
    // Implementation of the IDroolsWorkingMemory interface
    // ------------------------------------------------------------------------

    /* (non-Javadoc)
     * @see org.ow2.jasmine.rules.api.IDroolsWorkingMemory#stop()
     */
    public void stop() {

        if (workingMemory != null) {
            workingMemory.halt();
        }
        started = false;
        if (thread != null) {
            thread.interrupt();
            thread = null;
        }
        workingMemory = null;

    }

    public FactHandle insert(final Object fact) {
        if (workingMemory != null) {
            return workingMemory.insert(fact);
        } else {
            return null;
        }
    }

    // ------------------------------------------------------------------------
    //
    // ------------------------------------------------------------------------

    /**
     * Set a directory to save generated files.
     *
     * @param dumpDir
     *            directory to save generated files
     */
    public void setDumpDir(final File dumpDir) {
        this.dumpDir = dumpDir;
    }

    // ------------------------------------------------------------------------
    // Protected methods
    // ------------------------------------------------------------------------

    /**
     * Create a new working memory for a given rule base.
     *
     * @param ruleBase
     *            rule base
     */
    protected void createWorkingMemory() {
        logger.log(Level.INFO, "Init the Working Memory...");
        workingMemory = ruleBase.newStatefulSession();
    }

    protected void startRuleExecutor() {
        started = true;
        thread = new Thread(new RuleExecutor());
        thread.start();
    }

    /**
     * Periodically fire rules.
     */
    private class RuleExecutor implements Runnable {
        /**
         * Execute the rules after the timeout
         */
        public void run() {
            Logger logger = Logger.getLogger(this.getClass().getName());
            while (started) {
                try {
                    Thread.sleep(timeout);
                } catch (InterruptedException e) {
                    logger.log(Level.INFO, "DroolsWorkingMemory stopped");
                }

                if (workingMemory != null) {
                    try {
                        logger.log(Level.INFO, "Execute the rules...");
                        workingMemory.fireAllRules();
                    } catch (FactException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    public RuleBase getRuleBase() {
        return ruleBase;
    }

}
